@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2016 Maxime DOYEN
+ *  Copyright (C) 1995-2018 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -46,10 +46,10 @@ da_transaction_clean(Transaction *item)
 {
 	if(item != NULL)
 	{
-		if(item->wording != NULL)
+		if(item->memo != NULL)
 		{
-			g_free(item->wording);
-			item->wording = NULL;
+			g_free(item->memo);
+			item->memo = NULL;
 		}
 		if(item->info != NULL)
 		{
@@ -63,7 +63,8 @@ da_transaction_clean(Transaction *item)
 			item->tags = NULL;
 		}
 
-		da_transaction_splits_free(item);
+		da_splits_free(item->splits);
+		item->flags &= ~(OF_SPLIT); //Flag that Splits are cleared
 
 		if(item->same != NULL)
 		{
@@ -74,7 +75,6 @@ da_transaction_clean(Transaction *item)
 }
 
 
-
 void
 da_transaction_free(Transaction *item)
 {
@@ -95,8 +95,6 @@ da_transaction_malloc(void)
 
 Transaction *da_transaction_copy(Transaction *src_txn, Transaction *dst_txn)
 {
-guint count;
-
 	DB( g_print("da_transaction_copy\n") );
 
 	da_transaction_clean (dst_txn);
@@ -104,17 +102,14 @@ guint count;
 	memmove(dst_txn, src_txn, sizeof(Transaction));
 	
 	//duplicate the string
-	dst_txn->wording = g_strdup(src_txn->wording);
+	dst_txn->memo = g_strdup(src_txn->memo);
 	dst_txn->info = g_strdup(src_txn->info);
 
 	//duplicate tags
-	dst_txn->tags = NULL;
-	count = transaction_tags_count(src_txn);
-	if(count > 0)
-		//1501962: we must also copy the final 0
-		dst_txn->tags = g_memdup(src_txn->tags, (count+1)*sizeof(guint32));
+	transaction_tags_clone(src_txn, dst_txn);
 
-	da_transaction_splits_clone(src_txn, dst_txn);
+	if (da_splits_clone(src_txn->splits, dst_txn->splits) > 0)
+		dst_txn->flags |= OF_SPLIT; //Flag that Splits are active
 
 	return dst_txn;
 }
@@ -133,8 +128,10 @@ Transaction *da_transaction_init_from_template(Transaction *txn, Archive *arc)
 	txn->kpay		= arc->kpay;
 	txn->kcat		= arc->kcat;
 	txn->kxferacc	= arc->kxferacc;
-	txn->wording	= g_strdup(arc->wording);
+	txn->memo       = g_strdup(arc->memo);
 	txn->info		= NULL;
+	if( da_splits_clone(arc->splits, txn->splits) > 0)
+		txn->flags |= OF_SPLIT; //Flag that Splits are active
 
 	return txn;
 }
@@ -143,24 +140,20 @@ Transaction *da_transaction_init_from_template(Transaction *txn, Archive *arc)
 Transaction *da_transaction_clone(Transaction *src_item)
 {
 Transaction *new_item = g_memdup(src_item, sizeof(Transaction));
-guint count;
 
 	DB( g_print("da_transaction_clone\n") );
 
 	if(new_item)
 	{
 		//duplicate the string
-		new_item->wording = g_strdup(src_item->wording);
+		new_item->memo = g_strdup(src_item->memo);
 		new_item->info = g_strdup(src_item->info);
 
 		//duplicate tags
-		new_item->tags = NULL;
-		count = transaction_tags_count(src_item);
-		if(count > 0)
-			//1501962: we must also copy the final 0
-			new_item->tags = g_memdup(src_item->tags, (count+1)*sizeof(guint32));
-
-		da_transaction_splits_clone(src_item, new_item);
+		transaction_tags_clone(src_item, new_item);
+		
+		if( da_splits_clone(src_item->splits, new_item->splits) > 0)
+			new_item->flags |= OF_SPLIT; //Flag that Splits are active
 
 	}
 	return new_item;
@@ -177,24 +170,56 @@ da_transaction_new(void)
 guint
 da_transaction_length(void)
 {
-	return g_list_length(GLOBALS->ope_list);
-}
+GList *lst_acc, *lnk_acc;
+guint count = 0;
 
-
-void da_transaction_destroy(GList *list)
-{
-GList *tmplist = g_list_first(list);
-
-	while (tmplist != NULL)
+	lst_acc = g_hash_table_get_values(GLOBALS->h_acc);
+	lnk_acc = g_list_first(lst_acc);
+	while (lnk_acc != NULL)
 	{
-	Transaction *item = tmplist->data;
-		da_transaction_free(item);
-		tmplist = g_list_next(tmplist);
+	Account *acc = lnk_acc->data;
+	
+		count += g_queue_get_length (acc->txn_queue);
+		lnk_acc = g_list_next(lnk_acc);
 	}
-	g_list_free(list);
+	g_list_free(lst_acc);
+	return count;
 }
 
 
+static void da_transaction_queue_free_ghfunc(Transaction *item, gpointer data)
+{
+	da_transaction_free (item);
+}
+
+
+void da_transaction_destroy(void)
+{
+GList *lacc, *list;
+
+	lacc = g_hash_table_get_values(GLOBALS->h_acc);
+	list = g_list_first(lacc);
+	while (list != NULL)
+	{
+	Account *acc = list->data;
+
+		g_queue_foreach(acc->txn_queue, (GFunc)da_transaction_queue_free_ghfunc, NULL);
+		list = g_list_next(list);
+	}
+	g_list_free(lacc);
+}
+
+
+static gint da_transaction_compare_datafunc(Transaction *a, Transaction *b, gpointer data)
+{
+	return ((gint)a->date - b->date);
+}
+
+
+void da_transaction_queue_sort(GQueue *queue)
+{
+	g_queue_sort(queue, (GCompareDataFunc)da_transaction_compare_datafunc, NULL);
+}
 
 
 static gint da_transaction_compare_func(Transaction *a, Transaction *b)
@@ -209,36 +234,44 @@ GList *da_transaction_sort(GList *list)
 }
 
 
-static void da_transaction_insert_memo(Transaction *item)
+gboolean da_transaction_insert_memo(Transaction *item)
 {
+gboolean retval = FALSE;
+
 	// append the memo if new
-	if( item->wording != NULL )
+	if( item->memo != NULL )
 	{
-		if( g_hash_table_lookup(GLOBALS->h_memo, item->wording) == NULL )
+		if( g_hash_table_lookup(GLOBALS->h_memo, item->memo) == NULL )
 		{
-			g_hash_table_insert(GLOBALS->h_memo, g_strdup(item->wording), NULL);
+			retval = g_hash_table_insert(GLOBALS->h_memo, g_strdup(item->memo), NULL);
 		}
 	}
+	return retval;
 }
 
 
 gboolean da_transaction_insert_sorted(Transaction *newitem)
 {
-GList *tmplist = g_list_first(GLOBALS->ope_list);
+Account *acc;
+GList *lnk_txn;
 
-	// find the breaking date
-	while (tmplist != NULL)
+	acc = da_acc_get(newitem->kacc);
+	if(!acc) 
+		return FALSE;
+	
+	lnk_txn = g_queue_peek_tail_link(acc->txn_queue);
+	while (lnk_txn != NULL)
 	{
-	Transaction *item = tmplist->data;
+	Transaction *item = lnk_txn->data;
 
-		if(item->date > newitem->date)
+		if(item->date <= newitem->date)
 			break;
-
-		tmplist = g_list_next(tmplist);
+		
+		lnk_txn = g_list_previous(lnk_txn);
 	}
 
-	// here we're at the insert point, let's insert our new txn just before
-	GLOBALS->ope_list = g_list_insert_before(GLOBALS->ope_list, tmplist, newitem);
+	// we're at insert point, insert after txn
+	g_queue_insert_after(acc->txn_queue, lnk_txn, newitem);
 
 	da_transaction_insert_memo(newitem);
 	return TRUE;
@@ -248,7 +281,15 @@ GList *tmplist = g_list_first(GLOBALS->ope_list);
 // nota: this is called only when loading xml file
 gboolean da_transaction_prepend(Transaction *item)
 {
-	GLOBALS->ope_list = g_list_prepend(GLOBALS->ope_list, item);
+Account *acc;
+
+	acc = da_acc_get(item->kacc);
+	//#1661279
+	if(!acc) 
+		return FALSE;
+	
+	item->kcur = acc->kcur;
+	g_queue_push_tail(acc->txn_queue, item);
 	da_transaction_insert_memo(item);
 	return TRUE;
 }
@@ -259,23 +300,33 @@ gboolean da_transaction_prepend(Transaction *item)
 static guint32
 da_transaction_get_max_kxfer(void)
 {
-guint32 max_key = 0;
+GList *lst_acc, *lnk_acc;
 GList *list;
-Transaction *item;
+guint32 max_key = 0;
 
 	DB( g_print("da_transaction_get_max_kxfer\n") );
 
-	list = g_list_first(GLOBALS->ope_list);
-	while (list != NULL)
+	lst_acc = g_hash_table_get_values(GLOBALS->h_acc);
+	lnk_acc = g_list_first(lst_acc);
+	while (lnk_acc != NULL)
 	{
-		item = list->data;
-		if( item->paymode == PAYMODE_INTXFER)
+	Account *acc = lnk_acc->data;
+
+		list = g_queue_peek_head_link(acc->txn_queue);
+		while (list != NULL)
 		{
-			if( item->kxfer > max_key)
-				max_key = item->kxfer;
+		Transaction *item = list->data;
+
+			if( item->paymode == PAYMODE_INTXFER )
+			{
+				max_key = MAX(max_key, item->kxfer);
+			}
+			list = g_list_next(list);
 		}
-		list = g_list_next(list);
+		
+		lnk_acc = g_list_next(lnk_acc);
 	}
+	g_list_free(lst_acc);
 
 	DB( g_print(" max_key : %d \n", max_key) );
 
@@ -286,7 +337,12 @@ Transaction *item;
 static void da_transaction_goto_orphan(Transaction *txn)
 {
 const gchar *oatn = "orphaned transactions";
-Account *acc;
+Account *ori_acc, *acc;
+gboolean found;
+
+	DB( g_print("\n[transaction] goto orphan\n") );
+
+	g_warning("txn consistency: moving to orphan %d '%s' %.2f", txn->date, txn->memo, txn->amount);
 
 	acc = da_acc_get_by_name((gchar *)oatn);
 	if(acc == NULL)
@@ -294,8 +350,21 @@ Account *acc;
 		acc = da_acc_malloc();
 		acc->name = g_strdup(oatn);
 		da_acc_append(acc);
+		DB( g_print(" - created orphan acc %d\n", acc->key) );
+	}
+
+	ori_acc = da_acc_get(txn->kacc);
+	if( ori_acc )
+	{
+		found = g_queue_remove(ori_acc->txn_queue, txn);
+		DB( g_print(" - found in origin ? %d\n", found) );
+		if(found)
+		{
+			txn->kacc = acc->key;
+			da_transaction_insert_sorted (txn);
+			DB( g_print("moved txn to %d\n", txn->kacc) );
+		}
 	}
-	txn->kacc = acc->key;
 }
 
 
@@ -304,7 +373,9 @@ void da_transaction_consistency(Transaction *item)
 Account *acc;
 Category *cat;
 Payee *pay;
-guint i, nbsplit;
+gint nbsplit;
+
+	DB( g_print("\n[transaction] consistency\n") );
 
 	// ensure date is between range
 	item->date = CLAMP(item->date, HB_MINDATE, HB_MAXDATE);
@@ -328,18 +399,10 @@ guint i, nbsplit;
 	}
 
 	// check split category #1340142
-	nbsplit = da_transaction_splits_count(item);
-	for(i=0;i<nbsplit;i++)
-	{
-	Split *split = item->splits[i];
-		cat = da_cat_get(split->kcat);
-		if(cat == NULL)
-		{
-			g_warning("txn consistency: fixed invalid split cat %d", split->kcat);
-			split->kcat = 0;
-		}
-	}
-	//# 1416624 empty category when split + add control on kcat for v5.0.1
+	split_cat_consistency(item->splits);
+
+	//# 1416624 empty category when split
+	nbsplit = da_splits_count(item->splits);
 	if(nbsplit > 0 && item->kcat > 0)
 	{
 		g_warning("txn consistency: fixed invalid cat on split txn");
@@ -358,7 +421,30 @@ guint i, nbsplit;
 
 	// reset dst acc for non xfer transaction
 	if( item->paymode != PAYMODE_INTXFER )
+	{
+		item->kxfer = 0;
 		item->kxferacc = 0;
+	}
+
+	// check dst account exists
+	if( item->paymode == PAYMODE_INTXFER )
+	{
+	gint tak = item->kxferacc;
+
+		item->kxferacc = ABS(tak);  //I crossed negative here one day
+		acc = da_acc_get(item->kxferacc);
+		if(acc == NULL)
+		{
+			g_warning("txn consistency: fixed invalid dst_acc %d", item->kxferacc);
+			da_transaction_goto_orphan(item);
+			item->kxfer = 0;
+			item->paymode = PAYMODE_XFER;
+			GLOBALS->changes_count++;
+		}
+	}
+
+	//#1628678 tags for internal xfer should be checked as well
+
 
 	//#1295877 ensure income flag is correctly set
 	item->flags &= ~(OF_INCOME);
@@ -382,7 +468,7 @@ Transaction *child;
 Account *acc;
 gchar swap;
 
-	DB( g_print("\n[transaction] transaction_xfer_create_child\n") );
+	DB( g_print("\n[transaction] xfer_create_child\n") );
 
 	if( ope->kxferacc > 0 )
 	{
@@ -393,8 +479,9 @@ gchar swap;
 
 		child->amount = -child->amount;
 		child->flags ^= (OF_INCOME);	// invert flag
-		//#1268026
-		child->status = TXN_STATUS_NONE;
+		//#1268026 #1690555
+		if( child->status != TXN_STATUS_REMIND )
+			child->status = TXN_STATUS_NONE;
 		//child->flags &= ~(OF_VALID);	// delete reconcile state
 
 		swap = child->kacc;
@@ -419,7 +506,7 @@ gchar swap;
 			DB( g_print(" + strong link to %d\n", ope->kxfer) );
 
 
-			DB( g_print(" + add transfer, %p\n", child) );
+			DB( g_print(" + add transfer, %p to acc %d\n", child, acc->key) );
 
 			da_transaction_insert_sorted(child);
 
@@ -436,47 +523,72 @@ static gboolean transaction_xfer_child_might(Transaction *stxn, Transaction *dtx
 {
 gboolean retval = FALSE;
 
+	//DB( g_print("\n[transaction] xfer_child_might\n") );
+
 	if(stxn == dtxn)
 		return FALSE;
 
-	if( 
-		stxn->date == dtxn->date &&
-	    //stxn->kxferacc == dtxn->kacc &&
-		stxn->kacc != dtxn->kacc &&
+	/*g_print("test\n");
+
+	g_print(" %d %d %d %f %d\n", 
+		stxn->kcur, stxn->date, stxn->kacc, ABS(stxn->amount), stxn->kxfer );
+
+
+	g_print(" %d %d %d %f %d\n", 
+		dtxn->kcur, dtxn->date, dtxn->kacc, ABS(dtxn->amount), dtxn->kxfer );
+	*/
+
+	if( stxn->kcur == dtxn->kcur &&
+	    stxn->date == dtxn->date &&
+	    //v5.1 make no sense: stxn->kxferacc == dtxn->kacc &&
+	    stxn->kacc != dtxn->kacc &&
 	    ABS(stxn->amount) == ABS(dtxn->amount) &&
 	    dtxn->kxfer == 0)
 	{
 		retval = TRUE;
 	}
 
+	//g_print(" return %d\n", retval);
 	return retval;
 }
 
 
 static GList *transaction_xfer_child_might_list_get(Transaction *ope)
 {
-GList *list;
-GList *matchlist = NULL;
+GList *lst_acc, *lnk_acc;
+GList *list, *matchlist = NULL;
 
-	DB( g_print("\n[transaction]xfer_get_potential_child\n") );
+	//DB( g_print("\n[transaction] xfer_child_might_list_get\n") );
 
-	//DB( g_print(" - search : %d %s %f %d=>%d\n", src->date, src->wording, src->amount, src->account, src->kxferacc) );
-
-	list = g_list_first(GLOBALS->ope_list);
-	while (list != NULL)
+	lst_acc = g_hash_table_get_values(GLOBALS->h_acc);
+	lnk_acc = g_list_first(lst_acc);
+	while (lnk_acc != NULL)
 	{
-	Transaction *item = list->data;
+	Account *acc = lnk_acc->data;
 
-		if( transaction_xfer_child_might(ope, item, 0) == TRUE )
+		if( !(acc->flags & AF_CLOSED) && (acc->key != ope->kacc) )
 		{
-			//DB( g_print(" - match : %d %s %f %d=>%d\n", item->date, item->wording, item->amount, item->account, item->kxferacc) );
-
-			matchlist = g_list_append(matchlist, item);
+			list = g_queue_peek_tail_link(acc->txn_queue);
+			while (list != NULL)
+			{
+			Transaction *item = list->data;
+	
+				// no need to go higher than src txn date
+				if(item->date < ope->date)
+					break;
+		
+				if( transaction_xfer_child_might(ope, item, 0) == TRUE )
+				{
+					//DB( g_print(" - match : %d %s %f %d=>%d\n", item->date, item->memo, item->amount, item->account, item->kxferacc) );
+					matchlist = g_list_append(matchlist, item);
+				}
+				list = g_list_previous(list);
+			}
 		}
-		list = g_list_next(list);
+		
+		lnk_acc = g_list_next(lnk_acc);
 	}
-
-	DB( g_print(" - found : %d\n", g_list_length(matchlist)) );
+	g_list_free(lst_acc);
 
 	return matchlist;
 }
@@ -487,27 +599,40 @@ void transaction_xfer_search_or_add_child(GtkWindow *parentwindow, Transaction *
 GList *matchlist;
 gint count;
 
-	DB( g_print("\n[transaction] transaction_xfer_search_or_add_child\n") );
+	DB( g_print("\n[transaction] xfer_search_or_add_child\n") );
 
 	matchlist = transaction_xfer_child_might_list_get(ope);
 	count = g_list_length(matchlist);
 
-	DB( g_print(" - found result is %d, switching\n", count) );
+	DB( g_print(" - found %d might match, switching\n", count) );
 
-	if(count <= 1 && manual == FALSE)
+	switch(count)
 	{
-		//we should create the child
-		transaction_xfer_create_child(ope);
-	}
-	else
-	{
-	Transaction *child;
-
-		child = ui_dialog_transaction_xfer_select_child(parentwindow, ope, matchlist);
-		if(child == NULL)
+		case 0:		//we should create the child
 			transaction_xfer_create_child(ope);
-		else
-			transaction_xfer_change_to_child(ope, child);
+			break;
+
+		//todo: maybe with just 1 match the user must choose ?
+		//#942346: bad idea so to no let the user confirm, so let hil confirm
+		/*
+		case 1:		//transform the transaction to a child transfer
+		{
+			GList *list = g_list_first(matchlist);
+			transaction_xfer_change_to_child(ope, list->data);
+			break;
+		}
+		*/
+
+		default:	//the user must choose himself
+		{
+		Transaction *child;
+
+			child = ui_dialog_transaction_xfer_select_child(ope, matchlist);
+			if(child == NULL)
+				transaction_xfer_create_child(ope);
+			else
+				transaction_xfer_change_to_child(ope, child);
+		}
 	}
 
 	g_list_free(matchlist);
@@ -519,15 +644,15 @@ Transaction *transaction_xfer_child_strong_get(Transaction *src)
 Account *dstacc;
 GList *list;
 
-	DB( g_print("\n[transaction] transaction_xfer_child_strong_get\n") );
+	DB( g_print("\n[transaction] xfer_child_strong_get\n") );
 
 	dstacc = da_acc_get(src->kxferacc);
 	if( !dstacc || src->kxfer <= 0 )
 		return NULL;
 
-	DB( g_print(" - search: %d %s %f %d=>%d\n", src->date, src->wording, src->amount, src->kacc, src->kxferacc) );
+	DB( g_print(" - search: %d %s %f %d=>%d - %d\n", src->date, src->memo, src->amount, src->kacc, src->kxferacc, src->kxfer) );
 
-	list = g_list_first(GLOBALS->ope_list);
+	list = g_queue_peek_tail_link(dstacc->txn_queue);
 	while (list != NULL)
 	{
 	Transaction *item = list->data;
@@ -540,10 +665,10 @@ GList *list;
 		 && item->kxfer == src->kxfer 
 		 && item != src )
 		{
-			DB( g_print(" - found : %d %s %f %d=>%d\n", item->date, item->wording, item->amount, item->kacc, item->kxferacc) );
+			DB( g_print(" - found : %d %s %f %d=>%d - %d\n", item->date, item->memo, item->amount, item->kacc, item->kxferacc, src->kxfer) );
 			return item;
 		}
-		list = g_list_next(list);
+		list = g_list_previous(list);
 	}
 	
 	DB( g_print(" - not found...\n") );
@@ -557,7 +682,10 @@ void transaction_xfer_change_to_child(Transaction *ope, Transaction *child)
 {
 Account *dstacc;
 
-	DB( g_print("\n[transaction] transaction_xfer_change_to_child\n") );
+	DB( g_print("\n[transaction] xfer_change_to_child\n") );
+
+	if(ope->kcur != child->kcur)
+		return;
 
 	ope->flags |= OF_CHANGED;
 	child->flags |= OF_CHANGED;
@@ -580,10 +708,24 @@ Account *dstacc;
 }
 
 
-void transaction_xfer_sync_child(Transaction *s_txn, Transaction *child)
+void transaction_xfer_child_sync(Transaction *s_txn, Transaction *child)
 {
+Account *acc;
 
-	DB( g_print("\n[transaction] transaction_xfer_sync_child\n") );
+	DB( g_print("\n[transaction] xfer_child_sync\n") );
+
+	if( child == NULL )
+	{
+		DB( g_print(" - no child found\n") );
+		return;
+	}
+
+	DB( g_print(" - found do sync\n") );
+
+	/* update acc flags */
+	acc = da_acc_get( child->kacc);
+	if(acc != NULL)
+		acc->flags |= AF_CHANGED;
 
 	account_balances_sub (child);
 
@@ -596,20 +738,33 @@ void transaction_xfer_sync_child(Transaction *s_txn, Transaction *child)
 	  child->flags |= (OF_INCOME);
 	child->kpay		= s_txn->kpay;
 	child->kcat		= s_txn->kcat;
-	if(child->wording)
-		g_free(child->wording);
-	child->wording	= g_strdup(s_txn->wording);
+	if(child->memo)
+		g_free(child->memo);
+	child->memo	= g_strdup(s_txn->memo);
 	if(child->info)
 		g_free(child->info);
 	child->info		= g_strdup(s_txn->info);
 
-	//#1252230 sync account also
-	child->kacc		= s_txn->kxferacc;
-	child->kxferacc = s_txn->kacc;
-
 	account_balances_add (child);
-	
-	//todo: synchronise tags here also ?
+
+	//#1252230 sync account also
+	//#1663789 idem after 5.1
+	//source changed: update child key (move of s_txn is done in external_edit)
+	if( s_txn->kacc != child->kxferacc )
+	{
+		child->kxferacc = s_txn->kacc;
+	}
+
+	//dest changed: move child & update child key
+	if( s_txn->kxferacc != child->kacc )
+	{
+		transaction_acc_move(child, child->kacc, s_txn->kxferacc);
+	}
+
+	//synchronise tags since 5.1
+	if(child->tags)
+		g_free(child->tags);
+	transaction_tags_clone (s_txn, child);
 
 }
 
@@ -618,54 +773,68 @@ void transaction_xfer_remove_child(Transaction *src)
 {
 Transaction *dst;
 
-	DB( g_print("\n[transaction] transaction_xfer_remove_child\n") );
+	DB( g_print("\n[transaction] xfer_remove_child\n") );
 
 	dst = transaction_xfer_child_strong_get( src );
-
-	DB( g_print(" -> return is %s, %p\n", dst->wording, dst) );
-
 	if( dst != NULL )
 	{
-		DB( g_print("deleting...") );
-		src->kxfer = 0;
-		src->kxferacc = 0;
-		account_balances_sub(dst);
-		GLOBALS->ope_list = g_list_remove(GLOBALS->ope_list, dst);
+	Account *acc = da_acc_get(dst->kacc);
+
+		if( acc != NULL )
+		{
+			DB( g_print("deleting...") );
+			account_balances_sub(dst);
+			g_queue_remove(acc->txn_queue, dst);
+			//#1419304 we keep the deleted txn to a trash stack	
+			//da_transaction_free (dst);
+			g_trash_stack_push(&GLOBALS->txn_stk, dst);
+
+			//#1691992
+			acc->flags |= AF_CHANGED;
+		}
 	}
+	
+	src->kxfer = 0;
+	src->kxferacc = 0;
 }
 
 
 // still useful for upgrade from < file v0.6 (hb v4.4 kxfer)
 Transaction *transaction_old_get_child_transfer(Transaction *src)
 {
+Account *acc;
 GList *list;
 
-	DB( g_print("\n[transaction] transaction_get_child_transfer\n") );
+	DB( g_print("\n[transaction] get_child_transfer\n") );
 
-	//DB( g_print(" search: %d %s %f %d=>%d\n", src->date, src->wording, src->amount, src->account, src->kxferacc) );
+	//DB( g_print(" search: %d %s %f %d=>%d\n", src->date, src->memo, src->amount, src->account, src->kxferacc) );
+	acc = da_acc_get(src->kxferacc);
 
-	list = g_list_first(GLOBALS->ope_list);
-	while (list != NULL)
+	if( acc != NULL )
 	{
-	Transaction *item = list->data;
-
-		// no need to go higher than src txn date
-		if(item->date > src->date)
-			break;
-
-		if( item->paymode == PAYMODE_INTXFER)
+		list = g_queue_peek_head_link(acc->txn_queue);
+		while (list != NULL)
 		{
-			if( src->date == item->date &&
-			    src->kacc == item->kxferacc &&
-			    src->kxferacc == item->kacc &&
-			    ABS(src->amount) == ABS(item->amount) )
-			{
-				//DB( g_print(" found : %d %s %f %d=>%d\n", item->date, item->wording, item->amount, item->account, item->kxferacc) );
+		Transaction *item = list->data;
 
-				return item;
+			// no need to go higher than src txn date
+			if(item->date > src->date)
+				break;
+
+			if( item->paymode == PAYMODE_INTXFER)
+			{
+				if( src->date == item->date &&
+					src->kacc == item->kxferacc &&
+					src->kxferacc == item->kacc &&
+					ABS(src->amount) == ABS(item->amount) )
+				{
+					//DB( g_print(" found : %d %s %f %d=>%d\n", item->date, item->memo, item->amount, item->account, item->kxferacc) );
+
+					return item;
+				}
 			}
+			list = g_list_next(list);
 		}
-		list = g_list_next(list);
 	}
 
 	DB( g_print(" not found...\n") );
@@ -677,26 +846,55 @@ GList *list;
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
 
-void transaction_add(Transaction *ope, GtkWidget *treeview, guint32 accnum)
+void transaction_remove(Transaction *ope)
 {
-Transaction *newope;
 Account *acc;
 
-	DB( g_print("\n[transaction] transaction add\n") );
-
 	//controls accounts valid (archive scheduled maybe bad)
 	acc = da_acc_get(ope->kacc);
 	if(acc == NULL) return;
 
+	account_balances_sub(ope);
+
+	if( ope->paymode == PAYMODE_INTXFER )
+	{
+		transaction_xfer_remove_child( ope );
+	}
+	
+	g_queue_remove(acc->txn_queue, ope);
+	acc->flags |= AF_CHANGED;
+	//#1419304 we keep the deleted txn to a trash stack	
+	//da_transaction_free(entry);
+	g_trash_stack_push(&GLOBALS->txn_stk, ope);
+}
+
+
+Transaction *transaction_add(Transaction *ope)
+{
+Transaction *newope;
+Account *acc;
+
+	DB( g_print("\n[transaction] transaction_add\n") );
+
+	//controls accounts valid (archive scheduled maybe bad)
+	acc = da_acc_get(ope->kacc);
+	if(acc == NULL) return NULL;
+
+	DB( g_print(" acc is '%s' %d\n", acc->name, acc->key) );
+
+	ope->kcur = acc->kcur;
+	
 	if(ope->paymode == PAYMODE_INTXFER)
 	{
 		acc = da_acc_get(ope->kxferacc);
-		if(acc == NULL) return;
+		if(acc == NULL) return NULL;
 		
 		// delete any splits
-		da_transaction_splits_free(ope);
+		da_splits_free(ope->splits);
+		ope->flags &= ~(OF_SPLIT); //Flag that Splits are cleared
 	}
 
+
 	//allocate a new entry and copy from our edited structure
 	newope = da_transaction_clone(ope);
 
@@ -716,16 +914,19 @@ Account *acc;
 
 		/* get the active account and the corresponding cheque number */
 		acc = da_acc_get( newope->kacc);
-		cheque = atol(newope->info);
+		if( acc != NULL )
+		{
+			cheque = atol(newope->info);
 
-		//DB( g_print(" -> should store cheque number %d to %d", cheque, newope->account) );
-		if( newope->flags & OF_CHEQ2 )
-		{
-			acc->cheque2 = MAX(acc->cheque2, cheque);
-		}
-		else
-		{
-			acc->cheque1 = MAX(acc->cheque1, cheque);
+			//DB( g_print(" -> should store cheque number %d to %d", cheque, newope->account) );
+			if( newope->flags & OF_CHEQ2 )
+			{
+				acc->cheque2 = MAX(acc->cheque2, cheque);
+			}
+			else
+			{
+				acc->cheque1 = MAX(acc->cheque1, cheque);
+			}
 		}
 	}
 
@@ -735,13 +936,10 @@ Account *acc;
 	{
 		acc->flags |= AF_ADDED;
 
-		DB( g_print(" + add normal %p\n", newope) );
+		DB( g_print(" + add normal %p to acc %d\n", newope, acc->key) );
 		//da_transaction_append(newope);
 		da_transaction_insert_sorted(newope);
 
-		if(treeview != NULL)
-			transaction_add_treeview(newope, treeview, accnum);
-
 		account_balances_add(newope);
 
 		if(newope->paymode == PAYMODE_INTXFER)
@@ -749,37 +947,42 @@ Account *acc;
 			transaction_xfer_search_or_add_child(NULL, newope, FALSE);
 		}
 	}
+	
+	return newope;
 }
 
 
-void transaction_add_treeview(Transaction *ope, GtkWidget *treeview, guint32 accnum)
+gboolean transaction_acc_move(Transaction *txn, guint32 okacc, guint32 nkacc)
 {
-GtkTreeModel *model;
-GtkTreeIter  iter;
-//GtkTreePath *path;
-//GtkTreeSelection *sel;
+Account *oacc, *nacc;
 
-	DB( g_print("\n[transaction] transaction add treeview\n") );
+	DB( g_print("\n[transaction] acc_move\n") );
 
-	if(ope->kacc == accnum)
+	if( okacc == nkacc )
+		return TRUE;
+
+	oacc = da_acc_get(okacc);
+	nacc = da_acc_get(nkacc);
+	if( oacc && nacc )
 	{
-		model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
-		gtk_list_store_append (GTK_LIST_STORE(model), &iter);
-
-		gtk_list_store_set (GTK_LIST_STORE(model), &iter,
-				LST_DSPOPE_DATAS, ope,
-				-1);
-
-		//activate that new line
-		//path = gtk_tree_model_get_path(model, &iter);
-		//gtk_tree_view_expand_to_path(GTK_TREE_VIEW(treeview), path);
-
-		//sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-		//gtk_tree_selection_select_iter(sel, &iter);
-
-		//gtk_tree_path_free(path);
-
+		account_balances_sub(txn);
+		if( g_queue_remove(oacc->txn_queue, txn) )
+		{
+			g_queue_push_tail(nacc->txn_queue, txn);
+			txn->kacc = nacc->key;
+			txn->kcur = nacc->kcur;
+			nacc->flags |= AF_CHANGED;
+			account_balances_add(txn);
+			return TRUE;
+		}
+		else
+		{
+			//ensure to keep txn into current account
+			txn->kacc = okacc;
+			account_balances_add(txn);
+		}
 	}
+	return FALSE;
 }
 
 
@@ -793,7 +996,7 @@ gboolean match = FALSE;
 	if(text == NULL)
 		return FALSE;
 	
-	//DB( g_print("search %s in %s\n", rul->name, ope->wording) );
+	//DB( g_print("search %s in %s\n", rul->name, ope->memo) );
 	if( searchtext != NULL )
 	{
 		if( exact == TRUE )
@@ -822,12 +1025,30 @@ gboolean match = FALSE;
 	return match;
 }
 
+static gboolean misc_regex_match(gchar *text, gchar *searchtext, gboolean exact)
+{
+gboolean match = FALSE;
+
+	if(text == NULL)
+		return FALSE;
+	
+	DB( g_print("- match RE %s in %s\n", searchtext, text) );
+	if( searchtext != NULL )
+	{
+		match = g_regex_match_simple(searchtext, text, ((exact == TRUE)?0:G_REGEX_CASELESS) | G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY );
+		if (match == TRUE) { DB( g_print(" found pattern '%s'\n", searchtext) ); }
+	}
+	return match;
+}
+
 
 static Assign *transaction_auto_assign_eval_txn(GList *l_rul, Transaction *txn)
 {
 Assign *rule = NULL;
 GList *list;
-	
+
+	DB( g_print("\n[transaction] auto_assign_eval_txn\n") );
+
 	DB( g_print("- eval every rules, and return the last that match\n") );
 
 	list = g_list_first(l_rul);
@@ -836,15 +1057,24 @@ GList *list;
 	Assign *rul = list->data;
 	gchar *text;
 
-		text = txn->wording;
+		text = txn->memo;
 		if(rul->field == 1) //payee
 		{
 		Payee *pay = da_pay_get(txn->kpay);
 			if(pay)
 				text = pay->name;
 		}
-		if( misc_text_match(text, rul->name, rul->flags & ASGF_EXACT))
-			rule = rul;
+		
+		if( !(rul->flags & ASGF_REGEX) )
+		{
+			if( misc_text_match(text, rul->text, rul->flags & ASGF_EXACT) )
+				rule = rul;
+		}
+		else
+		{
+			if( misc_regex_match(text, rul->text, rul->flags & ASGF_EXACT) )
+				rule = rul;
+		}
 
 		list = g_list_next(list);
 	}
@@ -857,6 +1087,8 @@ static Assign *transaction_auto_assign_eval(GList *l_rul, gchar *text)
 {
 Assign *rule = NULL;
 GList *list;
+
+	DB( g_print("\n[transaction] auto_assign_eval\n") );
 	
 	DB( g_print("- eval every rules, and return the last that match\n") );
 
@@ -867,8 +1099,16 @@ GList *list;
 
 		if( rul->field == 0 )   //memo
 		{
-			if( misc_text_match(text, rul->name, rul->flags & ASGF_EXACT))
-				rule = rul;
+			if( !(rul->flags & ASGF_REGEX) )
+			{
+				if( misc_text_match(text, rul->text, rul->flags & ASGF_EXACT) )
+					rule = rul;
+			}
+			else
+			{
+				if( misc_regex_match(text, rul->text, rul->flags & ASGF_EXACT) )
+					rule = rul;
+			}
 		}
 		list = g_list_next(list);
 	}
@@ -883,7 +1123,7 @@ GList *l_ope;
 GList *l_rul;
 gint changes = 0;
 
-	DB( g_print("\n[transaction] transaction_auto_assign\n") );
+	DB( g_print("\n[transaction] auto_assign\n") );
 
 	l_rul = g_hash_table_get_values(GLOBALS->h_rul);
 
@@ -893,7 +1133,7 @@ gint changes = 0;
 	Transaction *ope = l_ope->data;
 	gboolean changed = FALSE; 
 
-		DB( g_print("- eval ope '%s' : acc=%d, pay=%d, cat=%d\n", ope->wording, ope->kacc, ope->kpay, ope->kcat) );
+		DB( g_print("- eval ope '%s' : acc=%d, pay=%d, cat=%d\n", ope->memo, ope->kacc, ope->kpay, ope->kcat) );
 
 		//#1215521: added kacc == 0
 		if( (kacc == ope->kacc || kacc == 0) )
@@ -903,7 +1143,7 @@ gint changes = 0;
 			rul = transaction_auto_assign_eval_txn(l_rul, ope);
 			if( rul != NULL )
 			{
-				if( (ope->kpay == 0 || (rul->flags & ASGF_OVWPAY)) && (rul->flags & ASGF_DOPAY) )
+				if( (ope->kpay == 0 && (rul->flags & ASGF_DOPAY)) || (rul->flags & ASGF_OVWPAY) )
 				{
 					if(ope->kpay != rul->kpay) { changed = TRUE; }
 					ope->kpay = rul->kpay;
@@ -911,17 +1151,28 @@ gint changes = 0;
 
 				if( !(ope->flags & OF_SPLIT) )
 				{
-					if( (ope->kcat == 0 || (rul->flags & ASGF_OVWCAT)) && (rul->flags & ASGF_DOCAT) )
+					if( (ope->kcat == 0 && (rul->flags & ASGF_DOCAT)) || (rul->flags & ASGF_OVWCAT) )
 					{
 						if(ope->kcat != rul->kcat) { changed = TRUE; }
 						ope->kcat = rul->kcat;
 					}
 				}
+
+				if( (ope->paymode == 0 && (rul->flags & ASGF_DOMOD)) || (rul->flags & ASGF_OVWMOD) )
+				{
+					//ugly hack - don't allow modify intxfer
+					if(ope->paymode != PAYMODE_INTXFER && rul->paymode != PAYMODE_INTXFER) 
+					{
+						if(ope->paymode != rul->paymode) { changed = TRUE; }
+						ope->paymode = rul->paymode;
+					}
+				}
+
 			}
 
 			if( ope->flags & OF_SPLIT )
 			{
-			guint i, nbsplit = da_transaction_splits_count (ope);
+			guint i, nbsplit = da_splits_count(ope->splits);
 
 				for(i=0;i<nbsplit;i++)
 				{
@@ -967,7 +1218,7 @@ transaction_tags_count(Transaction *ope)
 guint count = 0;
 guint32 *ptr = ope->tags;
 
-	DB( g_print("(transaction_tags_count)\n") );
+	//DB( g_print("\n[transaction] tags_count\n") );
 	
 	if( ope->tags == NULL )
 		return 0;
@@ -979,9 +1230,18 @@ guint32 *ptr = ope->tags;
 }
 
 
+void transaction_tags_clone(Transaction *src_txn, Transaction *dst_txn)
+{
+guint count;
 
-
-
+	dst_txn->tags = NULL;
+	count = transaction_tags_count(src_txn);
+	if(count > 0)
+	{
+		//1501962: we must also copy the final 0
+		dst_txn->tags = g_memdup(src_txn->tags, (count+1)*sizeof(guint32));
+	}
+}
 
 guint
 transaction_tags_parse(Transaction *ope, const gchar *tagstring)
@@ -990,7 +1250,7 @@ gchar **str_array;
 guint count, i;
 Tag *tag;
 
-	DB( g_print("(transaction_tags_parse)\n") );
+	DB( g_print("\n[transaction] tags_parse\n") );
 
 	DB( g_print(" - tagstring='%s'\n", tagstring) );
 
@@ -1044,7 +1304,7 @@ gchar **str_array;
 gchar *tagstring;
 Tag *tag;
 
-	DB( g_print("transaction_tags_tostring\n") );
+	DB( g_print("\n[transaction] tags_tostring\n") );
 
 	DB( g_print(" -> tags at=%p\n", ope->tags) );
 
