@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2016 Maxime DOYEN
+ *  Copyright (C) 1995-2018 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -20,13 +20,12 @@
 #ifndef __LIST_UPCOMING__H__
 #define __LIST_UPCOMING__H__
 
-GtkWidget *create_list_upcoming(void);
 
 enum
 {
 	LST_DSPUPC_DATAS,
 	LST_DSPUPC_PAYEE,
-	LST_DSPUPC_WORDING,
+	LST_DSPUPC_MEMO,
 	LST_DSPUPC_EXPENSE,
 	LST_DSPUPC_INCOME,
 	LST_DSPUPC_ACCOUNT,
@@ -36,4 +35,8 @@ enum
 	NUM_LST_DSPUPC
 };
 
-#endif
\ No newline at end of file
+
+GtkWidget *create_list_upcoming(void);
+
+
+#endif
