@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2016 Maxime DOYEN
+ *  Copyright (C) 1995-2018 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -287,27 +287,24 @@ guint32 catkey;
 	// get the category key
 	catkey = ui_cat_comboboxentry_get_key(GTK_COMBO_BOX(data->PO_cat));
 	
-	/* do nothing if no transaction or cat is 0 */
-	if( g_list_length(GLOBALS->ope_list) == 0 || catkey == 0 )
+	/* do nothing if cat is 0 */
+	if( catkey == 0 )
 		goto end1;
 
 	DB( g_print(" -> active cat is %d\n", catkey) );
 
+	g_queue_free (data->txn_queue);
+	data->txn_queue = hbfile_transaction_get_partial(data->filter->mindate, data->filter->maxdate);
+
 	// collect transactions
 	// the purpose here is to collect all cat transaction
 	// and precompute some datas
-	list = g_list_first(GLOBALS->ope_list);
+	list = g_queue_peek_head_link(data->txn_queue);
 	while (list != NULL)
 	{
 	Transaction *ope = list->data;
 	Category *cat;
 	CarCost *item;
-	Account *acc;
-
-		acc = da_acc_get(ope->kacc);
-		if(acc == NULL) goto next1;
-		if((acc->flags & (AF_CLOSED|AF_NOREPORT))) goto next1;
-		if((ope->status == TXN_STATUS_REMIND)) goto next1;
 
 		// eval normal transaction
 		if(!(ope->flags & OF_SPLIT))
@@ -317,18 +314,20 @@ guint32 catkey;
 			{
 				item = da_vehiclecost_malloc();
 				item->date = ope->date;
-				item->wording = ope->wording;
-				item->amount = ope->amount;
+				item->memo = ope->memo;
+				// get amount in base currency
+				//item->amount = ope->amount;
+				item->amount = hb_amount_base(ope->amount, ope->kcur);
 
-				item = repvehicle_eval_memofield(item, ope->wording);
+				item = repvehicle_eval_memofield(item, ope->memo);
 				data->vehicle_list = g_list_append(data->vehicle_list, item);
-				DB( g_print(" -> store acc=%d '%s' %.2f\n", ope->kacc, ope->wording, ope->amount) );
+				DB( g_print(" -> store acc=%d '%s' %.2f\n", ope->kacc, ope->memo, ope->amount) );
 			}
 		}
 		// eval split transaction
 		else
 		{
-		guint i, nbsplit = da_transaction_splits_count(ope);
+		guint i, nbsplit = da_splits_count(ope->splits);
 		Split *split;
 
 			DB( g_print(" -> nb split %d\n", nbsplit) );
@@ -344,8 +343,10 @@ guint32 catkey;
 				{
 					item = da_vehiclecost_malloc();
 					item->date = ope->date;
-					item->wording = split->memo;
-					item->amount = split->amount;
+					item->memo = split->memo;
+					// get amount in base currency
+					//item->amount = split->amount;
+					item->amount = hb_amount_base(split->amount, ope->kcur);
 
 					item = repvehicle_eval_memofield(item, split->memo);
 					data->vehicle_list = g_list_append(data->vehicle_list, item);
@@ -354,7 +355,6 @@ guint32 catkey;
 			}
 		}
 
-next1:
 		list = g_list_next(list);
 	}
 
@@ -403,16 +403,11 @@ guint lastmeter = 0;
 	gdouble centkm;
 	guint distbyvol;
 	gdouble trn_amount;
-	//Account *acc;
 
 		if( (item->date >= data->filter->mindate) && (item->date <= data->filter->maxdate) )
 		{
-			// get amount in base currency
-			//acc = da_acc_get(item->ope->kacc);
-			//trn_amount = to_base_amount(item->ope->amount, acc->kcur);
 			trn_amount = item->amount;
 
-
 			if( item->meter == 0 )
 			{
 				data->total_misccost += trn_amount;
@@ -468,7 +463,7 @@ guint lastmeter = 0;
 
 				distbyvol = 0;
 				if(centkm != 0)
-					distbyvol = arrondi((1/centkm)*100, 0);
+					distbyvol = hb_amount_round((1/centkm)*100, 0);
 
 				DB( g_print(" + pf=%.2f pd=%d :: dbv=%d\n", partial_fuel, partial_dist, distbyvol) );
 
@@ -478,7 +473,7 @@ guint lastmeter = 0;
 
 				gtk_list_store_set (GTK_LIST_STORE(model), &iter,
 					LST_CAR_DATE    , item->date,
-					LST_CAR_WORDING , item->wording,
+					LST_CAR_MEMO    , item->memo,
 					LST_CAR_METER   , item->meter,
 					LST_CAR_FUEL    , item->fuel,
 					LST_CAR_PRICE   , ABS(trn_amount) / item->fuel,
@@ -530,15 +525,15 @@ guint lastmeter = 0;
 
 		// 100km fuelcost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_avera[CAR_RES_FUELCOST]), data->total_fuelcost * coef, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_FUELCOST]), data->total_fuelcost * coef, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_FUELCOST]), data->total_fuelcost * coef, GLOBALS->kcur, GLOBALS->minor);
 
 		// 100km other cost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_avera[CAR_RES_OTHERCOST]), data->total_misccost * coef, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_OTHERCOST]), data->total_misccost * coef, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_OTHERCOST]), data->total_misccost * coef, GLOBALS->kcur, GLOBALS->minor);
 
 		// 100km cost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_avera[CAR_RES_TOTALCOST]), (data->total_fuelcost + data->total_misccost) * coef, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_TOTALCOST]), (data->total_fuelcost + data->total_misccost) * coef, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_avera[CAR_RES_TOTALCOST]), (data->total_fuelcost + data->total_misccost) * coef, GLOBALS->kcur, GLOBALS->minor);
 
 
 		// row 2 is for total
@@ -561,15 +556,15 @@ guint lastmeter = 0;
 
 		// total fuelcost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_total[CAR_RES_FUELCOST]), data->total_fuelcost, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_FUELCOST]), data->total_fuelcost, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_FUELCOST]), data->total_fuelcost, GLOBALS->kcur, GLOBALS->minor);
 
 		// total other cost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_total[CAR_RES_OTHERCOST]), data->total_misccost, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_OTHERCOST]), data->total_misccost, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_OTHERCOST]), data->total_misccost, GLOBALS->kcur, GLOBALS->minor);
 
 		// total cost
 		//hb_label_set_colvaluecurr(GTK_LABEL(data->LA_total[CAR_RES_TOTALCOST]), data->total_fuelcost + data->total_misccost, GLOBALS->kcur);
-		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_TOTALCOST]), data->total_fuelcost + data->total_misccost, GLOBALS->minor);
+		hb_label_set_colvalue(GTK_LABEL(data->LA_total[CAR_RES_TOTALCOST]), data->total_fuelcost + data->total_misccost, GLOBALS->kcur, GLOBALS->minor);
 
 
 }
@@ -606,6 +601,7 @@ static void repvehicle_setup(struct repvehicle_data *data)
 {
 	DB( g_print("(repvehicle) setup\n") );
 
+	data->txn_queue = g_queue_new ();
 	data->vehicle_list = NULL;
 
 	data->filter = da_filter_malloc();
@@ -648,6 +644,8 @@ struct WinGeometry *wg;
 
 	DB( g_print("(repvehicle) dispose\n") );
 
+	g_queue_free (data->txn_queue);
+	
 	da_vehiclecost_destroy(data->vehicle_list);
 
 	da_filter_free(data->filter);
@@ -679,7 +677,7 @@ GtkWidget *label, *widget, *table;
 gint row, col;
 GtkUIManager *ui;
 GtkActionGroup *actions;
-GtkAction *action;
+//GtkAction *action;
 GError *error = NULL;
 
 	data = g_malloc0(sizeof(struct repvehicle_data));
@@ -689,7 +687,7 @@ GError *error = NULL;
 
 	//disable define windows
 	GLOBALS->define_off++;
-	ui_mainwindow_update(GLOBALS->mainwindow, GINT_TO_POINTER(2));
+	ui_mainwindow_update(GLOBALS->mainwindow, GINT_TO_POINTER(UF_SENSITIVE));
 
     /* create window, etc */
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -697,6 +695,7 @@ GError *error = NULL;
 
 	//store our window private data
 	g_object_set_data(G_OBJECT(window), "inst_data", (gpointer)data);
+	DB( g_print(" - new window=%p, inst_data=%p\n", window, data) );
 
 	gtk_window_set_title (GTK_WINDOW (window), _("Vehicle cost report"));
 
@@ -734,7 +733,7 @@ GError *error = NULL;
 	gtk_grid_attach (GTK_GRID (table), widget, 2, row, 1, 1);
 
 	row++;
-	widget = gtk_check_button_new_with_mnemonic (_("_Minor currency"));
+	widget = gtk_check_button_new_with_mnemonic (_("Euro _minor"));
 	data->CM_minor = widget;
 	gtk_grid_attach (GTK_GRID (table), widget, 2, row, 1, 1);
 
@@ -770,7 +769,7 @@ GError *error = NULL;
     gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
 	//ui manager
-	actions = gtk_action_group_new ("Account");
+	actions = gtk_action_group_new ("default");
 
 	//as we use gettext
    	gtk_action_group_set_translation_domain(actions, GETTEXT_PACKAGE);
@@ -779,8 +778,8 @@ GError *error = NULL;
 	gtk_action_group_add_actions (actions, entries, n_entries, data);
 
 	/* set which action should have priority in the toolbar */
-	action = gtk_action_group_get_action(actions, "Refresh");
-	g_object_set(action, "is_important", TRUE, NULL);
+	//action = gtk_action_group_get_action(actions, "Refresh");
+	//g_object_set(action, "is_important", TRUE, NULL);
 
 
 	ui = gtk_ui_manager_new ();
@@ -1001,8 +1000,7 @@ gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
 
 	if( value )
 	{
-		mystrfmon(buf, G_ASCII_DTOSTR_BUF_SIZE-1, value, GLOBALS->minor);
-		//hb_strfmon(buf, G_ASCII_DTOSTR_BUF_SIZE-1, value, GLOBALS->kcur);
+		hb_strfmon(buf, G_ASCII_DTOSTR_BUF_SIZE-1, value, GLOBALS->kcur, GLOBALS->minor);
 
 		color = get_normal_color_amount(value);
 
@@ -1093,7 +1091,7 @@ GtkTreeViewColumn  *column;
 	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), PREFS->rules_hint);
+	gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (view), PREFS->grid_lines);
 
 	/* column date */
 	column = gtk_tree_view_column_new();
@@ -1108,7 +1106,7 @@ GtkTreeViewColumn  *column;
 
 /*
 	LST_CAR_DATE,
-	LST_CAR_WORDING,
+	LST_CAR_MEMO,
 	LST_CAR_METER,
 	LST_CAR_FUEL,
 	LST_CAR_PRICE,
@@ -1118,14 +1116,14 @@ GtkTreeViewColumn  *column;
 
 */
 
-	/* column: Wording */
+	/* column: Memo */
 /*
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(column, _("Wording"));
+	gtk_tree_view_column_set_title(column, _("Memo"));
 	gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
-	gtk_tree_view_column_add_attribute(column, renderer, "text", LST_CAR_WORDING);
+	gtk_tree_view_column_add_attribute(column, renderer, "text", LST_CAR_MEMO);
 	//gtk_tree_view_column_set_cell_data_func(column, renderer, repvehicle_text_cell_data_function, NULL, NULL);
 */
 
