@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2015 Maxime DOYEN
+ *  Copyright (C) 1995-2018 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -32,6 +32,7 @@
 
 /* our global datas */
 extern struct HomeBank *GLOBALS;
+extern struct Preferences *PREFS;
 
 
 gchar *CYA_CAT_TYPE[] = { 
@@ -133,6 +134,25 @@ gchar *name;
 	return -1;
 }
 
+
+Category
+*ui_cat_comboboxentry_get(GtkComboBox *entry_box)
+{
+Category *item = NULL;
+gchar *name;
+
+	DB( g_print ("ui_cat_comboboxentry_get_key()\n") );
+
+	name = (gchar *)gtk_entry_get_text(GTK_ENTRY (gtk_bin_get_child(GTK_BIN (entry_box))));
+	if(name == NULL)
+		return NULL;
+
+	item = da_cat_get_by_fullname(name);
+
+	return item;
+}
+
+
 gboolean
 ui_cat_comboboxentry_set_active(GtkComboBox *entry_box, guint32 key)
 {
@@ -221,11 +241,13 @@ gchar type;
 
 	if( ( item->key != ctx->except_key ) )
 	{
-		type = (item->flags & GF_INCOME) ? '+' : '-';
-
-		fullname = da_cat_get_fullname(item);
 		pitem = da_cat_get(item->parent);
 
+		type = (item->flags & GF_INCOME) ? '+' : '-';
+		fullname = da_cat_get_fullname(item);
+		sortname = NULL;
+		name = NULL;
+		
 		//DB( g_print ("cat combo populate [%d:%d] %s\n", item->parent, item->key, fullname) );
 
 		if(item->key == 0)
@@ -242,12 +264,15 @@ gchar type;
 			}
 			else
 			{
-				name = g_strdup_printf(" %c %s", type, item->name);
-				sortname = g_strdup_printf("%s_%s", pitem->name, item->name);
+				if(pitem)
+				{
+					name = g_strdup_printf(" %c %s", type, item->name);
+					sortname = g_strdup_printf("%s_%s", pitem->name, item->name);
+				}
 			}
 		}
 		
-		hb_string_replace_space(sortname);
+		hb_string_replace_char(' ', sortname);
 		
 		//gtk_list_store_append (GTK_LIST_STORE(ctx->model), &iter);
 		//gtk_list_store_set (GTK_LIST_STORE(ctx->model), &iter,
@@ -371,6 +396,10 @@ gint style;
 	    "style", style,
 		"text", name,
 		NULL);
+
+	//leak
+	g_free(name);
+
 }
 
 
@@ -451,11 +480,13 @@ GtkCellRenderer    *renderer;
 
     completion = gtk_entry_completion_new ();
     gtk_entry_completion_set_model (completion, GTK_TREE_MODEL(store));
-	g_object_set(completion, "text-column", LST_CMBCAT_FULLNAME, NULL);
+	//g_object_set(completion, "text-column", LST_CMBCAT_FULLNAME, NULL);
 	gtk_entry_completion_set_match_func(completion, ui_cat_comboboxentry_completion_func, NULL, NULL);
-	gtk_entry_completion_set_minimum_key_length(completion, 2);
+	//gtk_entry_completion_set_minimum_key_length(completion, 2);
 
-	renderer = gtk_cell_renderer_text_new ();
+	gtk_entry_completion_set_text_column(completion, 1);
+
+	/*renderer = gtk_cell_renderer_text_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion), renderer, TRUE);
 	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (completion), renderer, "text", LST_CMBCAT_FULLNAME, NULL);
 
@@ -463,6 +494,7 @@ GtkCellRenderer    *renderer;
 					    renderer,
 					    ui_cat_comboboxentry_test,
 					    NULL, NULL);
+	*/
 
 	// dothe same for combobox
 
@@ -499,38 +531,62 @@ ui_cat_listview_fixed_toggled (GtkCellRendererToggle *cell,
 	       gchar                 *path_str,
 	       gpointer               data)
 {
-  GtkTreeModel *model = (GtkTreeModel *)data;
-  GtkTreeIter  iter;
-  GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
-  gboolean fixed;
+GtkTreeModel *model = (GtkTreeModel *)data;
+GtkTreeIter  iter, child;
+GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
+gboolean fixed;
+gint n_child;
 
-  /* get toggled iter */
-  gtk_tree_model_get_iter (model, &iter, path);
-  gtk_tree_model_get (model, &iter, LST_DEFCAT_TOGGLE, &fixed, -1);
+	/* get toggled iter */
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, LST_DEFCAT_TOGGLE, &fixed, -1);
 
-  /* do something with the value */
-  fixed ^= 1;
+	/* do something with the value */
+	fixed ^= 1;
 
-  /* set new value */
-  gtk_tree_store_set (GTK_TREE_STORE (model), &iter, LST_DEFCAT_TOGGLE, fixed, -1);
+	/* set new value */
+	gtk_tree_store_set (GTK_TREE_STORE (model), &iter, LST_DEFCAT_TOGGLE, fixed, -1);
 
-  /* clean up */
-  gtk_tree_path_free (path);
+	/* propagate to child */
+	n_child = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(model), &iter);
+	gtk_tree_model_iter_children (GTK_TREE_MODEL(model), &child, &iter);
+	while(n_child > 0)
+	{
+		gtk_tree_store_set (GTK_TREE_STORE (model), &child, LST_DEFCAT_TOGGLE, fixed, -1);
+
+		n_child--;
+		gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &child);
+	}
+	
+	/* clean up */
+	gtk_tree_path_free (path);
 }
 
+
 static gint
 ui_cat_listview_compare_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata)
 {
-gint retval = 0;
+gint sortcol = GPOINTER_TO_INT(userdata);
 Category *entry1, *entry2;
-
+gint retval = 0;
+	
 	gtk_tree_model_get(model, a, LST_DEFCAT_DATAS, &entry1, -1);
 	gtk_tree_model_get(model, b, LST_DEFCAT_DATAS, &entry2, -1);
 
-	retval = (entry1->flags & GF_INCOME) - (entry2->flags & GF_INCOME);
-	if(!retval)
-	{
-		retval = hb_string_utf8_compare(entry1->name, entry2->name);
+    switch (sortcol)
+    {
+		case LST_DEFCAT_SORT_NAME:
+			retval = (entry1->flags & GF_INCOME) - (entry2->flags & GF_INCOME);
+			if(!retval)
+			{
+				retval = hb_string_utf8_compare(entry1->name, entry2->name);
+			}
+			break;
+		case LST_DEFCAT_SORT_USED:
+			retval = entry1->usage_count - entry2->usage_count;
+			break;
+		default:
+			g_return_val_if_reached(0);
 	}
     return retval;
 }
@@ -580,6 +636,28 @@ gchar *string;
 
 }
 
+
+static void
+ui_cat_listview_count_cell_data_function (GtkTreeViewColumn *col,
+				GtkCellRenderer *renderer,
+				GtkTreeModel *model,
+				GtkTreeIter *iter,
+				gpointer user_data)
+{
+Category *entry;
+gchar buffer[256];
+
+	gtk_tree_model_get(model, iter, LST_DEFCAT_DATAS, &entry, -1);
+	if(entry->usage_count > 0)
+	{
+		g_snprintf(buffer, 256-1, "%d", entry->usage_count);
+		g_object_set(renderer, "text", buffer, NULL);
+	}
+	else
+		g_object_set(renderer, "text", "", NULL);
+}
+
+
 /* = = = = = = = = = = = = = = = = */
 
 
@@ -928,7 +1006,7 @@ static gboolean ui_cat_listview_search_equal_func (GtkTreeModel *model,
 
 
 GtkWidget *
-ui_cat_listview_new(gboolean withtoggle)
+ui_cat_listview_new(gboolean withtoggle, gboolean withcount)
 {
 GtkTreeStore *store;
 GtkWidget *treeview;
@@ -947,7 +1025,8 @@ GtkTreeViewColumn	*column;
 	treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
-	//gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
+	gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (treeview), PREFS->grid_lines);
+
 
 	// column 1: toggle
 	if( withtoggle == TRUE )
@@ -963,19 +1042,45 @@ GtkTreeViewColumn	*column;
 	}
 
 	// column 2: name
-	column = gtk_tree_view_column_new();
 	renderer = gtk_cell_renderer_text_new ();
+	g_object_set(renderer, 
+		"ellipsize", PANGO_ELLIPSIZE_END,
+	    "ellipsize-set", TRUE,
+	    NULL);
+
+	column = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(column, _("Name"));
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func(column, renderer, ui_cat_listview_text_cell_data_function, GINT_TO_POINTER(LST_DEFCAT_NAME), NULL);
+	gtk_tree_view_column_set_alignment (column, 0.5);
+	gtk_tree_view_column_set_min_width(column, HB_MINWIDTH_LIST*2);
+	gtk_tree_view_column_set_sort_column_id (column, LST_DEFCAT_SORT_NAME);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
 
+	if( withcount == TRUE )
+	{
+		column = gtk_tree_view_column_new();
+		gtk_tree_view_column_set_title(column, _("Usage"));
+		renderer = gtk_cell_renderer_text_new ();
+		g_object_set(renderer, "xalign", 0.5, NULL);
+		gtk_tree_view_column_pack_start(column, renderer, TRUE);
+		gtk_tree_view_column_set_cell_data_func(column, renderer, ui_cat_listview_count_cell_data_function, GINT_TO_POINTER(LST_DEFCAT_DATAS), NULL);
+		gtk_tree_view_column_set_alignment (column, 0.5);
+		gtk_tree_view_column_set_sort_column_id (column, LST_DEFCAT_SORT_USED);
+		gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
+	}
+
+	
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(treeview), ui_cat_listview_search_equal_func, NULL, NULL);
 
 	// treeview attribute
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(treeview), FALSE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(treeview), withcount);
 
-	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), LST_DEFCAT_DATAS, ui_cat_listview_compare_func, NULL, NULL);
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), LST_DEFCAT_DATAS, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), LST_DEFCAT_SORT_NAME, ui_cat_listview_compare_func, GINT_TO_POINTER(LST_DEFCAT_SORT_NAME), NULL);
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), LST_DEFCAT_SORT_USED, ui_cat_listview_compare_func, GINT_TO_POINTER(LST_DEFCAT_SORT_USED), NULL);
+	
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), LST_DEFCAT_SORT_NAME, GTK_SORT_ASCENDING);
 
 	return treeview;
 }
@@ -1022,6 +1127,36 @@ gchar *result = g_new0 (gchar, length+1);
 }
 
 
+static void
+ui_cat_manage_dialog_delete_unused( GtkWidget *widget, gpointer user_data)
+{
+struct ui_cat_manage_dialog_data *data = user_data;
+gboolean result;
+
+	//data = g_object_get_data(G_OBJECT(gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_WINDOW)), "inst_data");
+
+	DB( g_print("(ui_cat_manage_dialog) delete unused - data %p\n", data) );
+
+	result = ui_dialog_msg_confirm_alert(
+			GTK_WINDOW(data->window),
+			_("Delete unused categories"),
+			_("Are you sure you want to permanently\ndelete unused categories?"),
+			_("_Delete")
+		);
+
+	if( result == GTK_RESPONSE_OK )
+	{
+	GtkTreeModel *model;	
+		
+		model = gtk_tree_view_get_model(GTK_TREE_VIEW(data->LV_cat));
+		gtk_tree_store_clear (GTK_TREE_STORE(model));
+
+		category_delete_unused();
+	
+		ui_cat_manage_populate_listview (data);
+	}
+}
+
 
 
 /**
@@ -1047,7 +1182,7 @@ gchar *error;
 		{
 			ui_dialog_msg_infoerror(GTK_WINDOW(data->window), GTK_MESSAGE_ERROR,
 					_("File format error"),
-					_("The csv file must contains the exact numbers of column,\nseparated by a semi-colon, please see the help for more details.")
+					_("The CSV file must contains the exact numbers of column,\nseparated by a semi-colon, please see the help for more details.")
 					);
 		}
 
@@ -1444,7 +1579,7 @@ gint result;
 		title = g_strdup_printf (
 			_("Are you sure you want to permanently delete '%s'?"), item->name);
 
-		if( category_is_used(item->key) == TRUE )
+		if( item->usage_count > 0 )
 		{
 			secondtext = _("This category is used.\n"
 			    "Any transaction using that category will be set to (no category)");
@@ -1751,7 +1886,14 @@ gint w, h, row;
 	menuitem = gtk_menu_item_new_with_mnemonic (_("E_xport CSV"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_cat_manage_dialog_save_csv), &data);
+
+	menuitem = gtk_separator_menu_item_new();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	
+	menuitem = gtk_menu_item_new_with_mnemonic (_("_Delete unused"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate", G_CALLBACK (ui_cat_manage_dialog_delete_unused), &data);
+
 	gtk_widget_show_all (menu);
 	
 	widget = gtk_menu_button_new();
@@ -1793,7 +1935,7 @@ gint w, h, row;
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwin), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scrollwin), HB_MINHEIGHT_LIST);
- 	treeview = ui_cat_listview_new(FALSE);
+ 	treeview = ui_cat_listview_new(FALSE, TRUE);
 	data.LV_cat = treeview;
 	gtk_container_add(GTK_CONTAINER(scrollwin), treeview);
 	gtk_widget_set_hexpand (scrollwin, TRUE);
@@ -1880,8 +2022,8 @@ gint w, h, row;
 	g_signal_connect (G_OBJECT (data.ST_name1), "activate", G_CALLBACK (ui_cat_manage_dialog_add), GINT_TO_POINTER(FALSE));
 	g_signal_connect (G_OBJECT (data.ST_name2), "activate", G_CALLBACK (ui_cat_manage_dialog_add), GINT_TO_POINTER(TRUE));
 
-	g_signal_connect(G_OBJECT(data.ST_name1), "insert_text", G_CALLBACK(ui_cat_manage_filter_text_handler), NULL);
-	g_signal_connect(G_OBJECT(data.ST_name2), "insert_text", G_CALLBACK(ui_cat_manage_filter_text_handler), NULL);
+	g_signal_connect(G_OBJECT(data.ST_name1), "insert-text", G_CALLBACK(ui_cat_manage_filter_text_handler), NULL);
+	g_signal_connect(G_OBJECT(data.ST_name2), "insert-text", G_CALLBACK(ui_cat_manage_filter_text_handler), NULL);
 
 
 	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data.LV_cat)), "changed", G_CALLBACK (ui_cat_manage_dialog_selection), NULL);
@@ -1895,6 +2037,7 @@ gint w, h, row;
 	g_signal_connect (G_OBJECT (data.BT_collapse), "clicked", G_CALLBACK (ui_cat_manage_dialog_collapse_all), NULL);
 
 	//setup, init and show window
+	category_fill_usage();
 	ui_cat_manage_dialog_setup(&data);
 	ui_cat_manage_dialog_update(data.LV_cat, NULL);
 
