@@ -1,5 +1,5 @@
 /*  HomeBank -- Free, easy, personal accounting for everyone.
- *  Copyright (C) 1995-2015 Maxime DOYEN
+ *  Copyright (C) 1995-2018 Maxime DOYEN
  *
  *  This file is part of HomeBank.
  *
@@ -22,6 +22,9 @@
 #include "ui-dialogs.h"
 #include "list_operation.h"
 
+#include "hb-currency.h"
+#include "ui-currency.h"
+
 
 /* = = = = = = = = = = */
 /* = = = = = = = = = = = = = = = = = = = = */
@@ -201,7 +204,7 @@ gint crow, row, count;
 	gtk_grid_attach (GTK_GRID (group_grid), label, 0, 0, 3, 1);
 
 	row = 1;
-	label = make_label(_("Account"), 0, 0.5);
+	label = make_label_widget(_("Account"));
 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
 	widget = make_label(NULL, 1.0, 0.5);
 	count = da_acc_length ();
@@ -210,7 +213,7 @@ gint crow, row, count;
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
 
 	row++;
-	label = make_label(_("Transaction"), 0, 0.5);
+	label = make_label_widget(_("Transaction"));
 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
 	widget = make_label(NULL, 1.0, 0.5);
 	count = da_transaction_length();
@@ -219,7 +222,7 @@ gint crow, row, count;
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
 
 	row++;
-	label = make_label(_("Payee"), 0, 0.5);
+	label = make_label_widget(_("Payee"));
 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
 	widget = make_label(NULL, 1.0, 0.5);
 	count = da_pay_length ();
@@ -228,7 +231,7 @@ gint crow, row, count;
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
 
 	row++;
-	label = make_label(_("Category"), 0, 0.5);
+	label = make_label_widget(_("Category"));
 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
 	widget = make_label(NULL, 1.0, 0.5);
 	count = da_cat_length ();
@@ -237,7 +240,7 @@ gint crow, row, count;
 	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
 
 	row++;
-	label = make_label(_("Assignment"), 0, 0.5);
+	label = make_label_widget(_("Assignment"));
 	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
 	widget = make_label(NULL, 1.0, 0.5);
 	count = da_asg_length ();
@@ -263,6 +266,154 @@ gint crow, row, count;
 
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
+struct dialog_currency_data
+{
+	GtkWidget   *window;
+	GtkWidget   *LB_currency;
+	GtkWidget   *BT_change;
+	Currency4217 *curfmt;
+};
+
+static void ui_dialog_upgrade_choose_currency_change_action(GtkWidget *widget, gpointer user_data)
+{
+struct dialog_currency_data *data = user_data;
+struct curSelectContext selectCtx;
+
+	data->curfmt = NULL;
+
+	ui_cur_select_dialog_new(GTK_WINDOW(data->window), CUR_SELECT_MODE_BASE, &selectCtx);
+	if( selectCtx.cur_4217 != NULL )
+	{
+	Currency4217 *curfmt;
+	gchar label[128];
+	gchar *name;
+	
+		curfmt = selectCtx.cur_4217;	
+		
+		DB( g_printf("- user selected: '%s' '%s'\n", curfmt->curr_iso_code, curfmt->name) );
+
+		data->curfmt = curfmt;
+
+		name = curfmt->name;
+
+		g_snprintf(label, 127, "%s - %s", curfmt->curr_iso_code, name);
+		gtk_label_set_text (GTK_LABEL(data->LB_currency), label);
+	}
+}
+
+
+static void ui_dialog_upgrade_choose_currency_fill(struct dialog_currency_data *data)
+{
+Currency *cur;
+gchar label[128];
+
+	data->curfmt = NULL;
+
+	cur = da_cur_get (GLOBALS->kcur);
+
+	g_snprintf(label, 127, "%s - %s", cur->iso_code, cur->name);
+	gtk_label_set_text (GTK_LABEL(data->LB_currency), label);
+}
+
+
+
+void ui_dialog_upgrade_choose_currency(void)
+{
+struct dialog_currency_data data;
+GtkWidget *dialog, *content_area, *content_grid, *group_grid;
+GtkWidget *label, *widget;
+gint crow, row;
+
+	dialog = gtk_dialog_new_with_buttons (_("Upgrade"),
+		GTK_WINDOW (GLOBALS->mainwindow),
+		0,
+		_("_Cancel"), GTK_RESPONSE_CANCEL,
+		_("_OK"), GTK_RESPONSE_ACCEPT,
+		NULL);
+
+	data.window = dialog;
+
+	widget = gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+	gtk_window_set_focus(GTK_WINDOW(dialog), widget);
+
+	content_area = gtk_dialog_get_content_area(GTK_DIALOG (dialog));
+
+	content_grid = gtk_grid_new();
+	gtk_grid_set_row_spacing (GTK_GRID (content_grid), SPACING_LARGE);
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(content_grid), GTK_ORIENTATION_VERTICAL);
+	gtk_container_set_border_width (GTK_CONTAINER(content_grid), SPACING_MEDIUM);
+	gtk_box_pack_start (GTK_BOX (content_area), content_grid, TRUE, TRUE, 0);
+
+	crow = 0;
+	// group :: file title
+    group_grid = gtk_grid_new ();
+	gtk_grid_set_row_spacing (GTK_GRID (group_grid), SPACING_MEDIUM);
+	gtk_grid_set_column_spacing (GTK_GRID (group_grid), SPACING_MEDIUM);
+	gtk_grid_attach (GTK_GRID (content_grid), group_grid, 0, crow++, 1, 1);
+
+	row = 0;
+	label = make_label(_("Select a base currency"), 0, 0);
+	gimp_label_set_attributes(GTK_LABEL(label), 
+		PANGO_ATTR_WEIGHT, PANGO_WEIGHT_BOLD, 
+		PANGO_ATTR_SCALE,  PANGO_SCALE_LARGE, 
+		-1);
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, row, 3, 1);
+
+	row++;
+	label = make_label(
+		_("Starting v5.1, HomeBank can manage several currencies\n" \
+		  "if the currency below is not correct, please change it:"), 0, 0);
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, row, 3, 1);
+
+	row++;
+	label = make_label_widget(_("Currency:"));
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, row, 1, 1);
+	widget = make_label (NULL, 0, 0.5);
+	data.LB_currency = widget;
+	gtk_grid_attach (GTK_GRID (group_grid), widget, 1, row, 1, 1);
+	widget = gtk_button_new_with_mnemonic (_("_Change"));
+	data.BT_change = widget;
+	gtk_grid_attach (GTK_GRID (group_grid), widget, 2, row, 1, 1);
+
+	g_signal_connect (G_OBJECT (data.BT_change), "clicked", G_CALLBACK (ui_dialog_upgrade_choose_currency_change_action), &data);
+
+
+	ui_dialog_upgrade_choose_currency_fill(&data);
+
+	gtk_widget_show_all(content_grid);
+
+	//wait for the user
+	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if(result == GTK_RESPONSE_ACCEPT)
+	{
+
+		if( data.curfmt != NULL )
+		{
+			hbfile_replace_basecurrency(data.curfmt);
+		}
+	}
+
+	// in any case set every accounts to base currency
+	GList *list;
+	list = g_hash_table_get_values(GLOBALS->h_acc);
+	while (list != NULL)
+	{
+	Account *acc = list->data;
+
+		account_set_currency(acc, GLOBALS->kcur);
+		list = g_list_next(list);
+	}
+	g_list_free(list);
+	
+	// cleanup and destroy
+	gtk_widget_destroy (dialog);
+
+}
+
+
+/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
+
 
 
 
@@ -300,7 +451,7 @@ gboolean retval;
 	{
 	gchar *tmpfilename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 	
-		*storage_ptr = hb_util_filename_new_with_extension(tmpfilename, "qif");
+		*storage_ptr = hb_filename_new_with_extension(tmpfilename, "qif");
 		g_free(tmpfilename);
 		retval = TRUE;
 	}
@@ -360,7 +511,7 @@ gchar *path;
 	
 		if( action == GTK_FILE_CHOOSER_ACTION_SAVE )
 		{
-			*storage_ptr = hb_util_filename_new_with_extension(tmpfilename, "csv");
+			*storage_ptr = hb_filename_new_with_extension(tmpfilename, "csv");
 			g_free(tmpfilename);
 		}
 		else
@@ -389,12 +540,12 @@ gboolean retval;
 
 	if( action == GTK_FILE_CHOOSER_ACTION_OPEN )
 	{
-		title = _("Open homebank file");
+		title = _("Open HomeBank file");
 		button = _("_Open");
 	}
 	else
 	{
-		title = _("Save homebank file as");
+		title = _("Save HomeBank file as");
 		button = _("_Save");
 	}
 
@@ -406,6 +557,7 @@ gboolean retval;
 					NULL);
 
 	ui_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), _("HomeBank files"), "*.[Xx][Hh][Bb]");
+	//ui_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), _("Backup files"), "*.[Bb][Aa][Kk]");
 	ui_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), _("All files"), "*");
 
 	if( action == GTK_FILE_CHOOSER_ACTION_OPEN )
@@ -523,35 +675,133 @@ GtkWidget *dialog = NULL;
 
 		gtk_dialog_set_default_response(GTK_DIALOG( dialog ), 2);
 
-	  result = gtk_dialog_run( GTK_DIALOG( dialog ) );
-	  gtk_widget_destroy( dialog );
+		result = gtk_dialog_run( GTK_DIALOG( dialog ) );
+		gtk_widget_destroy( dialog );
 
-	  if(result == 1 || result == GTK_RESPONSE_DELETE_EVENT)
+		if(result == 1 || result == GTK_RESPONSE_DELETE_EVENT)
 		{
-	  	retval = FALSE;
+			retval = FALSE;
 		}
 		else
 		{
-	  if(result == 2)
-	  {
-		DB( g_print(" + should quick save %s\n", GLOBALS->xhb_filepath) );
-		homebank_save_xml(GLOBALS->xhb_filepath);
-	  }
+			if(result == 2)
+			{
+				DB( g_print(" + should quick save %s\n", GLOBALS->xhb_filepath) );
+				//todo: should migrate this
+				//#1720377 also backup 
+				homebank_file_ensure_xhb(NULL);
+				homebank_backup_current_file();
+				homebank_save_xml(GLOBALS->xhb_filepath);
+			}
 		}
 
-
-
 	}
 	return retval;
 }
 
 
+/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 
 
+gint ui_dialog_export_pdf(GtkWindow *parent, gchar **storage_ptr)
+{
+GtkWidget *dialog, *content_area, *content_grid, *group_grid;
+GtkWidget *label, *widget, *BT_folder, *ST_name;
+gchar *tmpstr;
+gint crow, row;
+
+	dialog = gtk_dialog_new_with_buttons (_("Export as PDF"),
+		GTK_WINDOW (parent),
+		0,
+		_("_Cancel"), GTK_RESPONSE_CANCEL,
+		_("Export as _PDF"), GTK_RESPONSE_ACCEPT,
+		NULL);
+
+	gtk_window_set_default_size (GTK_WINDOW(dialog), HB_MINWIDTH_LIST, -1);
+	
+	content_area = gtk_dialog_get_content_area(GTK_DIALOG (dialog));
+
+	content_grid = gtk_grid_new();
+	gtk_grid_set_row_spacing (GTK_GRID (content_grid), SPACING_LARGE);
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(content_grid), GTK_ORIENTATION_VERTICAL);
+	gtk_container_set_border_width (GTK_CONTAINER(content_grid), SPACING_MEDIUM);
+	gtk_box_pack_start (GTK_BOX (content_area), content_grid, TRUE, TRUE, 0);
+
+	crow = 0;
+	// group :: file title
+    group_grid = gtk_grid_new ();
+	gtk_grid_set_row_spacing (GTK_GRID (group_grid), SPACING_SMALL);
+	gtk_grid_set_column_spacing (GTK_GRID (group_grid), SPACING_MEDIUM);
+	gtk_grid_attach (GTK_GRID (content_grid), group_grid, 0, crow++, 1, 1);
+
+	row = 0;
+	widget = gtk_image_new_from_icon_name (ICONNAME_WARNING, GTK_ICON_SIZE_DIALOG);
+	gtk_grid_attach (GTK_GRID (group_grid), widget, 0, row, 1, 1);
+	label = gtk_label_new("This feature is still in development state,\n(maybe not stable) so use it at your own risk!");
+	gtk_grid_attach (GTK_GRID (group_grid), label, 1, row, 1, 1);
+
+	row++;
+	label = make_label_widget(_("Folder:"));
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, row, 1, 1);
+	BT_folder = gtk_file_chooser_button_new (_("Pick a Folder"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	gtk_grid_attach (GTK_GRID (group_grid), BT_folder, 1, row, 1, 1);
+
+	row++;
+	label = make_label_widget(_("Filename:"));
+	gtk_grid_attach (GTK_GRID (group_grid), label, 0, row, 1, 1);
+	ST_name = make_string (label);
+	gtk_grid_attach (GTK_GRID (group_grid), ST_name, 1, row, 1, 1);
+
+
+	//setup
+	tmpstr = g_path_get_dirname(*storage_ptr);
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(BT_folder), tmpstr);
+	g_free(tmpstr);
+	
+	tmpstr = g_path_get_basename(*storage_ptr);
+	gtk_entry_set_text(GTK_ENTRY(ST_name), tmpstr);
+	g_free(tmpstr);
+
+
+	gtk_widget_show_all(content_grid);
+
+	//wait for the user
+	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if(result == GTK_RESPONSE_ACCEPT)
+	{
+	gchar *hostname;
+	//#300380 fixed export path problem (was always the export of preference)
+	//not to be used -- gchar *nufolder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(BT_folder));
+	gchar *urifolder  = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(BT_folder));
+	gchar *nufolder   = g_filename_from_uri(urifolder, &hostname, NULL);
+	gchar *nufilename = hb_filename_new_with_extension((gchar *)gtk_entry_get_text (GTK_ENTRY(ST_name)), "pdf");	
+
+		g_free(*storage_ptr);
+		*storage_ptr = g_build_filename(nufolder, nufilename, NULL);
+
+		g_free(nufilename);
+		g_free(nufolder);
+		g_free(urifolder);
+	}
+
+	// cleanup and destroy
+	gtk_widget_destroy (dialog);
+
+	return result;
+}
+
+
+
+
+
+/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
+
 struct xfer_data
 {
 	GtkWidget   *window;
 	GtkWidget	*radio[2];
+	GtkWidget	*srctreeview;
 	GtkWidget	*treeview;
 };
 
@@ -590,19 +840,17 @@ static void ui_dialog_transaction_xfer_select_child_selection_cb(GtkTreeSelectio
 }
 
 
-Transaction *ui_dialog_transaction_xfer_select_child(GtkWidget *treeview, GList *matchlist)
+Transaction *ui_dialog_transaction_xfer_select_child(Transaction *stxn, GList *matchlist)
 {
 struct xfer_data data;
-GtkWidget *parentwindow, *window, *content, *mainvbox, *vbox, *sw, *label;
+GtkWidget *window, *content, *mainvbox, *vbox, *sw, *label, *LB_several;
 GtkTreeModel		 *newmodel;
 GtkTreeIter			 newiter;
 Transaction *retval = NULL;
 
-	parentwindow = gtk_widget_get_ancestor(GTK_WIDGET(treeview), GTK_TYPE_WINDOW);
-
 	window = gtk_dialog_new_with_buttons (
-    			NULL,
-    			GTK_WINDOW (parentwindow),
+    			_("Select among possible transactions..."),
+    			GTK_WINDOW (GLOBALS->mainwindow),
 			    0,
 			    _("_Cancel"),
 			    GTK_RESPONSE_REJECT,
@@ -613,7 +861,7 @@ Transaction *retval = NULL;
 	g_object_set_data(G_OBJECT(window), "inst_data", (gpointer)&data);
 	data.window = window;
 
-		//gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
+	//gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
 	gtk_window_set_default_size (GTK_WINDOW (window), 800, 494);
 
 	content = gtk_dialog_get_content_area(GTK_DIALOG (window));
@@ -621,18 +869,19 @@ Transaction *retval = NULL;
 		gtk_box_pack_start (GTK_BOX (content), mainvbox, TRUE, TRUE, 0);
 		gtk_container_set_border_width (GTK_CONTAINER (mainvbox), SPACING_SMALL);
 
-		gtk_window_set_title (GTK_WINDOW (window), _("Select among possible transactions..."));
+	// source listview
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_ETCHED_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	//gtk_widget_set_size_request(sw, -1, HB_MINWIDTH_LIST/2);
 
-		label = make_label(_(
-		"HomeBank has found some transaction that may be " \
-		"the associated transaction for the internal transfer."), 0.0, 0.5
-		);
-	gimp_label_set_attributes (GTK_LABEL (label),
-                             PANGO_ATTR_SCALE,  PANGO_SCALE_SMALL,
-                             -1);
-	gtk_box_pack_start (GTK_BOX (mainvbox), label, FALSE, FALSE, SPACING_SMALL);
+	data.srctreeview = create_list_transaction(LIST_TXN_TYPE_DETAIL, PREFS->lst_ope_columns);
+	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(data.srctreeview)), GTK_SELECTION_NONE);
+	gtk_container_add (GTK_CONTAINER (sw), data.srctreeview);
+	gtk_box_pack_start (GTK_BOX (mainvbox), sw, TRUE, TRUE, 0);
 
 
+	// actions 
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, SPACING_SMALL);
 	gtk_box_pack_start (GTK_BOX (mainvbox), vbox, FALSE, TRUE, SPACING_SMALL);
 
@@ -640,34 +889,50 @@ Transaction *retval = NULL;
 	gimp_label_set_attributes(GTK_LABEL(label), PANGO_ATTR_WEIGHT, PANGO_WEIGHT_BOLD, -1);
     gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
-
 	data.radio[0] = gtk_radio_button_new_with_label (NULL, _("create a new transaction"));
 	gtk_box_pack_start (GTK_BOX (vbox), data.radio[0], FALSE, FALSE, 0);
 
 	data.radio[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (data.radio[0]), _("select an existing transaction"));
 	gtk_box_pack_start (GTK_BOX (vbox), data.radio[1], FALSE, FALSE, 0);
 
+
+	label = make_label(_(
+		"HomeBank has found some transaction that may be " \
+		"the associated transaction for the internal transfer."), 0.0, 0.5
+		);
+	LB_several = label;
+	gimp_label_set_attributes (GTK_LABEL (label),
+                             PANGO_ATTR_SCALE,  PANGO_SCALE_SMALL,
+                             -1);
+	gtk_box_pack_start (GTK_BOX (mainvbox), label, FALSE, FALSE, SPACING_SMALL);
+
+	// target listview
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-	data.treeview = create_list_transaction(LIST_TXN_TYPE_BOOK, PREFS->lst_ope_columns);
+	data.treeview = create_list_transaction(LIST_TXN_TYPE_DETAIL, PREFS->lst_ope_columns);
 	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(data.treeview)), GTK_SELECTION_SINGLE);
 	gtk_container_add (GTK_CONTAINER (sw), data.treeview);
 	gtk_box_pack_start (GTK_BOX (mainvbox), sw, TRUE, TRUE, 0);
 
-	g_signal_connect (data.radio[0], "toggled", G_CALLBACK (ui_dialog_transaction_xfer_select_child_cb), NULL);
-	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data.treeview)), "changed", G_CALLBACK (ui_dialog_transaction_xfer_select_child_selection_cb), NULL);
+	/* populate source */
+	if( stxn != NULL )
+	{
+		newmodel = gtk_tree_view_get_model(GTK_TREE_VIEW(data.srctreeview));
+		gtk_list_store_clear (GTK_LIST_STORE(newmodel));
 	
-	/* populate */
+		gtk_list_store_append (GTK_LIST_STORE(newmodel), &newiter);
+
+		gtk_list_store_set (GTK_LIST_STORE(newmodel), &newiter,
+		LST_DSPOPE_DATAS, stxn,
+		-1);
+	}
+
+	/* populate target */
 	newmodel = gtk_tree_view_get_model(GTK_TREE_VIEW(data.treeview));
-
-
-
-
 	gtk_list_store_clear (GTK_LIST_STORE(newmodel));
 
-
 	GList *tmplist = g_list_first(matchlist);
 	while (tmplist != NULL)
 	{
@@ -680,42 +945,58 @@ Transaction *retval = NULL;
 			LST_DSPOPE_DATAS, tmp,
 			-1);
 
-		//DB( g_print(" - fill: %s %.2f %x\n", item->wording, item->amount, (unsigned int)item->same) );
+		//DB( g_print(" - fill: %s %.2f %x\n", item->memo, item->amount, (unsigned int)item->same) );
 
 		tmplist = g_list_next(tmplist);
 	}
 
+
+	g_signal_connect (data.radio[0], "toggled", G_CALLBACK (ui_dialog_transaction_xfer_select_child_cb), NULL);
+	g_signal_connect (gtk_tree_view_get_selection(GTK_TREE_VIEW(data.treeview)), "changed", G_CALLBACK (ui_dialog_transaction_xfer_select_child_selection_cb), NULL);
+
+	gtk_widget_show_all(mainvbox);
+
 	//initialize
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data.radio[1]), TRUE);
+	gtk_widget_set_sensitive (data.radio[1], TRUE);
+	if( g_list_length (matchlist) <= 0 )
+	{
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data.radio[0]), TRUE);
+		gtk_widget_set_sensitive (data.radio[1], FALSE);
+		gtk_widget_set_visible (LB_several, FALSE);
+	}
+	else
+	{
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data.radio[1]), TRUE);
+		gtk_widget_set_visible (LB_several, TRUE);
+	}
+	
+	ui_dialog_transaction_xfer_select_child_cb(data.radio[0], NULL);
 
+	//wait for the user
+	gint result = gtk_dialog_run (GTK_DIALOG (window));
 
-		gtk_widget_show_all(mainvbox);
+	if(result == GTK_RESPONSE_ACCEPT)
+	{
+	gboolean bnew;
 
-		//wait for the user
-		gint result = gtk_dialog_run (GTK_DIALOG (window));
-
-		if(result == GTK_RESPONSE_ACCEPT)
+		bnew = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data.radio[0]));
+		if( bnew == FALSE )
 		{
-		gboolean bnew;
+		GtkTreeSelection *selection;
+		GtkTreeModel		 *model;
+		GtkTreeIter			 iter;
 
-			bnew = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data.radio[0]));
-			if( bnew == FALSE)
+			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data.treeview));
+			if (gtk_tree_selection_get_selected(selection, &model, &iter))
 			{
-			GtkTreeSelection *selection;
-			GtkTreeModel		 *model;
-			GtkTreeIter			 iter;
-
-				selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data.treeview));
-				if (gtk_tree_selection_get_selected(selection, &model, &iter))
-				{
-					gtk_tree_model_get(model, &iter, LST_DSPOPE_DATAS, &retval, -1);
-				}
+				gtk_tree_model_get(model, &iter, LST_DSPOPE_DATAS, &retval, -1);
 			}
-
 		}
 
-		// cleanup and destroy
-		gtk_widget_destroy (window);
+	}
+
+	// cleanup and destroy
+	gtk_widget_destroy (window);
 
 	return retval;
 }
